import sys, os
import numpy as np
from scipy.optimize import curve_fit
from matplotlib import pyplot as plt

import BioNetTopology

sys.setrecursionlimit(10000)

def recursivelyAddNodesToCluster(index, cluster, remaining, nConnections):

	# Have we been here?
	if(index not in cluster):

		# Add to cluster
		cluster.add(index)

		# Remove from global list
		remaining.remove(index)

		# Get next node
		for nIndex in nConnections[index]:
			cluster, remaining = recursivelyAddNodesToCluster(nIndex, cluster, remaining, nConnections)

	return cluster, remaining

#
# Graph shit
#

SMALL = 8
MEDIUM = 10
LARGE = 12

#
# Data
#

# Constants
numNodes = 10000

# Variables
vfs = [0.005, 0.01, 0.05, 0.07, 0.1]
ns = [4,6,8,14]
rates = [1000]
radii = [1.0, 2.5]
#numRuns = 3

#vfs = [0.07]
#ns = [8]
#rates = [1000]
#radii = [2.5]

numRuns = 3

# Loop over variables
for radius in radii:
	for rate in rates:

		for vf in vfs:

			for s in ns:

				for nr in range(numRuns):

					# Get the topology
					tpFname = "../output/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.bntop" % (numNodes, s, radius, rate, vf, nr)
					if(not os.path.exists(tpFname)):
						continue

					top = BioNetTopology.BioNetTopology()
					top.loadFinalFrame(tpFname)

					# Connect all the nodes
					top.frame[-1].buildNodeConnections(top.numNodes)

					# Get a set of all node indices
					nodesRemaining = set(range(top.numNodes))

					# Initialise clusters
					clusters = []

					# Search while there are still nodes unaccounted for
					while(len(nodesRemaining) != 0):

						# New cluster
						cluster = set([])

						# Get a central node
						centralNode = nodesRemaining.pop()

						# Readd for recursion
						nodesRemaining.add(centralNode)

						# Recursively add
						cluster, nodesRemaining = recursivelyAddNodesToCluster(centralNode, cluster, nodesRemaining, top.frame[-1].nConnections)
						
						# Add to clusters
						clusters.append(cluster)

					# Info
					numClusters = len(clusters)
					lengths = [len(c) for c in clusters]
					minCluster = min(lengths)
					maxCluster = max(lengths)

					print("R: %.2f, Ns: %d, vf: %f" % (radius, s, vf))
					print("Num Clusters: %d, Max: %d, Min: %d\n" % (numClusters, maxCluster, minCluster))

					# Write clusters to file
					oFname = "../results/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d_Clusters.dat" % (numNodes, s, radius, rate, vf, nr)
					fout = open(oFname, "w")
					fout.write("numClusters %d\n" % (len(clusters)))
					for cluster in clusters:
						c = list(cluster)
						fout.write("%d" % (c[0]))
						for i in c[1:]:
							fout.write(",%d" % (i))
						fout.write("\n")
					fout.close()
