import sys, os
import numpy as np
from matplotlib import pyplot as plt

import BioNetInput, BioNetTrajectory

#
# Graph shit
#
SMALL = 8
MEDIUM = 10
LARGE = 12

#
# Data
#

# Constants
numNodes = 10000

# Variables
vfs = [0.005,0.01,0.05,0.07,0.1]
ns = [4,6,8,14]
rates = [1000]

radii = [1.0,2.5]


numRuns = 3

# Loop over variables
for radius in radii:
	for rate in rates:

		sIndex = 0
		for s in ns:

			# Get a figure for plotting fd calculation traces
			fig = plt.figure(figsize=(3.2,3.2))
			vIndex = 0
			for vf in vfs:

				# Output
				oFname = "../results/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKinetics_FractalDimensions.dat" % (numNodes, s, radius, rate, vf)
				fout = open(oFname, "w")

				# Build data sets
				minSize = np.inf

				# Counts frames
				traj = BioNetTrajectory.BioNetTrajectory()
				for nr in range(numRuns):

					# Load script
					tFname = "../output/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.bntrj" % (numNodes, s, radius, rate, vf, nr)
					if(not os.path.exists(tFname)):	
						continue


					numFrames = traj.countFrames(tFname)
					
					if numFrames < minSize:
						minSize = numFrames
			

				# Read the final frame
				for nr in range(numRuns):

					# Load scripts
					iFname = "../input/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.xml" % (numNodes, s, radius, rate, vf, nr)
					tFname = "../output/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.bntrj" % (numNodes, s, radius, rate, vf, nr)
					if(not os.path.exists(tFname)):	
						continue

					# Files
					traj = BioNetTrajectory.BioNetTrajectory()
					traj.loadFrame(tFname, minSize-1)
#					traj.loadFirstFrame(tFname)
					inp = BioNetInput.BioNetInput()
					inp.load(iFname)

					if inp.box.BC=="PBC":
						periodic = True
					else:
						periodic = False

					rads = np.array([radius for n in inp.node])
					thresh = np.min(rads) *  1.0 / 5.0
#					thresh = np.min(rads)

					# Calc FD
#					pwfd, pwx01, data = traj.frame[0].calculateFractalDimension_BoxCount(inp.box.dimension, threshold=thresh, radius = rads, periodic=periodic)
					fdPair, data = traj.frame[0].calculateFractalDimension_BoxCount(inp.box.dimension, thresh, 3, radius = rads, periodic=periodic)

					# Write all relevent parameters
					fractalLength = inp.box.dimension[0] / np.power(2, data[3][2])
					fractalLengthErr = fractalLength * np.sqrt(data[4][2][2]) * np.log(2)
					
					fout.write("%f %f %f %f " % (fdPair[0], fdPair[1], fractalLength, fractalLengthErr))

				fout.write("\n")
				fout.close()

				# Plot everything (fit and all) (we haven't taken averages here)
				XData = data[0]
				YData = data[1]
				fitFunc = data[2]
				popt = data[3]
				plt.plot(XData, YData, "xC%d" % (vIndex), label=r"$f_v = $%0.4f" % (vf))
				plt.plot(XData, fitFunc(XData, *popt), "--C%d" % (vIndex))

				vIndex += 1

			# Prettify
			plt.xlabel(r"log $_2$ (Num Boxes 1D)", fontsize=MEDIUM)
			plt.ylabel(r"log $_2$ (NumBoxes Populated)", fontsize=MEDIUM)
			plt.legend(loc=0, fontsize=SMALL)
			plt.xlim(plt.xlim()[0], 12)
			plt.ylim(plt.ylim()[0], 35)
			plt.tight_layout()
			oFname2 = "../results/N%dSites%02dRadius%05.2fRate%5.2fKinetics_BoxCount.png" % (numNodes, s, radius, rate)
			plt.savefig(oFname2, dpi=100)
			plt.clf()

			sIndex += 1


