import sys, os
import numpy as np

import BioNetInput, BioNetTrajectory



#
# Data
#

# Constants
numNodes = 10000

# Variables
vfs = [0.005, 0.01, 0.05, 0.07, 0.1]
#vfs = [0.01, 0.1]
ns = [4,6,8,14]
#ns = [6]
rates = [1000]

radii = [1.0,2.5]
#radii = [1.0]
numRuns = 3

# Loop over variables
for radius in radii:
	rIndex = 0
	for rate in rates:
		vIndex = 0
		for vf in vfs:

			sIndex = 0
			for s in ns:

				# Output data files
				oFname = "../results/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKinetics_RDFTraceLOL.dat" % (numNodes, s, radius, rate, vf)
				fout = open(oFname, "w")

				# Build data sets
				for nr in range(numRuns):

					# Input file
					iFname = "../input/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.xml" % (numNodes, s, radius, rate, vf, nr)
					inp = BioNetInput.BioNetInput()
					inp.load(iFname)

					# Load script
					tFname = "../output/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.bntrj" % (numNodes, s, radius, rate, vf, nr)
		
					if(not os.path.exists(tFname)):	
						continue

					traj = BioNetTrajectory.BioNetTrajectory()
					traj.loadFinalFrame(tFname)

					# RDF for entire system (400 particles for statistics, 8 protein diameters for graph)
					nP = 1000
					mR = 6 * np.max([n.radius for n in inp.node])
					dr = 0.05 * np.max([n.radius for n in inp.node])
					rs, rdf, rdfErr = traj.frame[0].calculateRadialDistributionFunction(inp, maxRadius=mR, numParticles=nP, dr=dr)

					# Check for nans
					if np.isnan(rdf[0]):
						rdf[0] = 0.0
						rdfErr[0] = 0.0

					# Write to file
					fout.write("%.3f" % (rs[0]))
					for r in rs[1:]:
						fout.write(",%.2f" % (r))
					fout.write("\n")

					fout.write("%.3f" % (rdf[0]))
					for r in rdf[1:]:
						fout.write(",%.2f" % (r))
					fout.write("\n")

					fout.write("%.3f" % (rdfErr[0]))
					for r in rdfErr[1:]:
						fout.write(",%.2f" % (r))
					fout.write("\n")
				fout.close()

					

				

				sIndex += 1

			vIndex += 1
		rIndex += 1		

