import sys, os
import numpy as np
from scipy.optimize import curve_fit
from matplotlib import pyplot as plt

import BioNetInput, BioNetTopology, BioNetTrajectory

#
# Graph shit
#

SMALL = 8
MEDIUM = 10
LARGE = 12

#
# Data
#

# Constants
numNodes = 10000
radius = 1.0

# Variables
vfs = [0.005, 0.01, 0.05, 0.07, 0.1]
ns = [4,6,8,14]
rates = [1000]

numRuns = 3

radii = [1.0,2.5]

# Figure
fig = plt.figure(figsize=(3.2,3.2))

# Loop over variables
for radius in radii:
	for rate in rates:

		for vf in vfs:

			sIndex = 0

			# Plot stuff
			maxCount = 6
			xTicks = []
			xTickLabels = []
			for s in ns:

				# Build data sets
				pairDist = [np.array([0 for i in range(max(ns))], dtype="float") for j in range(numRuns)]
				minSize = np.inf
				exists = False

				# Counts frames
				traj = BioNetTrajectory.BioNetTrajectory()
				for nr in range(numRuns):

					# Load script
					tFname = "../output/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.bntrj" % (numNodes, s, radius, rate, vf, nr)
					if(not os.path.exists(tFname)):	
						continue


					numFrames = traj.countFrames(tFname)
					
					if numFrames < minSize:
						minSize = numFrames

				# Read the final frame
				for nr in range(numRuns):

					# Load scripts
					iFname = "../input/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.xml" % (numNodes, s, radius, rate, vf, nr)
					tFname = "../output/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.bntop" % (numNodes, s, radius, rate, vf, nr)
					if(not os.path.exists(tFname)):	
						continue

					inp = BioNetInput.BioNetInput()
					inp.load(iFname)

					top = BioNetTopology.BioNetTopology()
					top.loadFrame(tFname, minSize-1)

					# Get data
					top.frame[0].buildNodeConnections(top.numNodes)
					for n in range(top.numNodes):

						# Loop over and count repeats
						top.frame[0].nConnections[n].sort()
						
						m = 0
						while m < len(top.frame[0].nConnections[n]) != 0:

							# Count copies
							nCount = top.frame[0].nConnections[n].count(top.frame[0].nConnections[n][m])

							# Update data and max pairing
							pairDist[nr][nCount - 1] += 1

							# Update m
							m += nCount


					# Flag something existing
					exists = True	

				# Check for existence
				if(not exists):
					continue

				# What is the max count? (also, we double counted, but transform to probabilities anyway)
				for nr in range(len(pairDist)):
					pairDist[nr] = pairDist[nr][:maxCount]
					pairDist[nr] /= sum(pairDist[nr])

				# We've overcounted

				# Average and errors
				meanTrace = np.mean(pairDist, axis=0)
				sigmaTrace = np.std(pairDist, axis=0)

				errTrace = sigmaTrace / np.sqrt(len(pairDist))

				# Add to plot
				xVals = np.array(range(maxCount)) + (sIndex * (maxCount + 1))
				xTicks.extend(list(xVals))
				xTickLabels.extend(np.array(range(maxCount)) + 1)
				plt.bar(xVals, meanTrace, yerr=errTrace, color="C%d" % (sIndex), label=r"$N_s = $%d" % (s))
				
				sIndex += 1

			# Prettify
			ax = plt.gca()
			ax.set_xticks(xTicks)
			ax.set_xticklabels(xTickLabels)
			plt.ylim(0,1.1)
			plt.xlabel("Bonds per Protein Pair", fontsize=MEDIUM)
			plt.ylabel("Probability", fontsize=MEDIUM)
			plt.legend(loc=0, fontsize=SMALL)

			plt.tight_layout()
			oFname = "../results/N%dRadius%05.2fRate%5.2fvf%0.4fKinetics_CoordinationPairings.png" % (numNodes, radius, rate, vf)
			plt.savefig(oFname, dpi=100)
			plt.clf()

