import sys, os
import numpy as np
from scipy.optimize import curve_fit
from matplotlib import pyplot as plt

import BioNetMeasurement

# Data extraction method
def calculateDataIndex(sI, vI, rI, numVars):

	# '8' because there are 8 values per set of parameters
	return 8 * (sI + numVars[-1] * (vI + numVars[-2] * rI ))

#
# Graph shit
#

SMALL = 8
MEDIUM = 10
LARGE = 12

#
# Data
#

# Constants
numNodes = 10000

# Variables
vfs = [0.005, 0.01, 0.05, 0.07, 0.1]
ns = [4,6,8,14]
rates = [1000]
radii = [1.0,2.5]

numRuns = 3


# Loop over variables
for radius in radii:

	# Multiple figs
	fig = [plt.figure(i, figsize=(3.2,3.2)) for i in range(3)]

	rIndex = 0
	for rate in rates:

		# Data input file
		iFname = "../results/N%dRadius%05.2fRate%5.2fKinetics_CoordinationTrace.dat" % (numNodes, radius, rate)

		# Get header info
		fin = open(iFname, "r")
		line = fin.readline()
		fin.close()

		numVars = [int(bit) for bit in line.split()[1::2]]

		# Get actual data
		data = np.genfromtxt(iFname, skip_header=1, delimiter=",")

		sIndex = 0
		for s in ns:

			t1 = [None for vf in vfs]
			t1Err = [None for vf in vfs]
			t2 = [None for vf in vfs]
			t2Err = [None for vf in vfs]

			CN1 = [None for vf in vfs]
			CN1Err = [None for vf in vfs]
			CN2 = [None for vf in vfs]
			CN2Err = [None for vf in vfs]

			CNinf = [None for vf in vfs]
			CNinfErr = [None for vf in vfs]

			vIndex = 0
			for vf in vfs:

				# Manual ignore due to lack of percolation
#				if(vf == 0.05 and radius == 2.5 and s == 6):
#					CN1 = CN1[:-1]
#					CN1Err = CN1Err[:-1]
#					CN2 = CN2[:-1]
#					CN2Err = CN2Err[:-1]

#					t1 = t1[:-1]
#					t1Err = t1Err[:-1]
#					t2 = t2[:-1]
#					t2Err = t2Err[:-1]
#					vIndex += 1
#					continue
#					pass

				CN1[vIndex] = data[calculateDataIndex(sIndex, vIndex, rIndex, numVars) + 0]
				CN1Err[vIndex] = data[calculateDataIndex(sIndex, vIndex, rIndex, numVars) + 1]
				CN2[vIndex] = data[calculateDataIndex(sIndex, vIndex, rIndex, numVars) + 4]
				CN2Err[vIndex] = data[calculateDataIndex(sIndex, vIndex, rIndex, numVars) + 5]

				t1[vIndex] = data[calculateDataIndex(sIndex, vIndex, rIndex, numVars) + 2]
				t1Err[vIndex] = data[calculateDataIndex(sIndex, vIndex, rIndex, numVars) + 3]
				t2[vIndex] = data[calculateDataIndex(sIndex, vIndex, rIndex, numVars) + 6]
				t2Err[vIndex] = data[calculateDataIndex(sIndex, vIndex, rIndex, numVars) + 7]

				# Summations
				if(np.isnan(CN1[vIndex])):
					CNinf[vIndex] = CN2[vIndex]
					CNinfErr[vIndex] = CN2Err[vIndex]
				elif(np.isnan(CN2[vIndex])):
					CNinf[vIndex] = CN1[vIndex]
					CNinfErr[vIndex] = CN1Err[vIndex]
				else:
					CNinf[vIndex] = CN1[vIndex] + CN2[vIndex]
					CNinfErr[vIndex] = CN1Err[vIndex] + CN2Err[vIndex]
				vIndex += 1

			# Plot trace for every sites
			plt.figure(0)
			plt.errorbar(vfs, CNinf, yerr=CNinfErr, fmt="-xC%d" % (sIndex))
			plt.plot(vfs, CNinf, "-C%d" % (sIndex), label=r"$N_s = $%d" % (s))

			plt.figure(1)
			plt.errorbar(vfs, CN1, yerr=CN1Err, fmt="-xC%d" % (sIndex))
			plt.errorbar(vfs, CN2, yerr=CN2Err, fmt="-oC%d" % (sIndex))
			plt.plot(vfs, CN1, "-C%d" % (sIndex), label=r"$N_s = $%d" % (s))

			plt.figure(2)
			plt.errorbar(vfs, t1, yerr=t1Err, fmt="-xC%d" % (sIndex))
			plt.errorbar(vfs, t2, yerr=t2Err, fmt="-oC%d" % (sIndex))
			plt.plot(vfs, t1, "-C%d" % (sIndex), label=r"$N_s = $%d" % (s))

			sIndex += 1

		# Prettify
		locs = [0.0,0.025,0.05,0.075,0.1]
		labels = [0.0,0.025,0.05,0.075,0.1]

		plt.figure(0)
		plt.xticks(locs,labels)
		plt.xlim(plt.xlim()[0], 0.11)
		plt.ylim(0,10)
		plt.xlabel(r"$f_v$", fontsize=MEDIUM)
		plt.ylabel("Average Coordination", fontsize=MEDIUM)

		if radius == 1.0:
			plt.legend(loc=0, fontsize=SMALL, ncol=2)
		else:
			plt.legend(loc=0, fontsize=SMALL, ncol=1)

		plt.tight_layout()
		oFname = "../results/N%dRadius%05.2fRate%5.2fKinetics_CoordinationTrend_yInf.png" % (numNodes, radius, rate)
		plt.savefig(oFname, dpi=100)
		plt.clf()

		plt.figure(1)

		plt.xticks(locs,labels)
		plt.xlim(plt.xlim()[0], 0.11)
		plt.ylim(0,10)

#		plt.plot([-1], [-1], "xk", label="Process 1")
#		plt.plot([-1], [-1], "ok", label="Process 2")

		plt.xlabel(r"$f_v$", fontsize=MEDIUM)
		plt.ylabel("Coordination Weights", fontsize=MEDIUM)

		if radius == 1.0:
			plt.legend(loc=0, fontsize=SMALL, ncol=1)
		else:
			plt.legend(loc=0, fontsize=SMALL, ncol=2)
		plt.tight_layout()
		oFname = "../results/N%dRadius%05.2fRate%5.2fKinetics_CoordinationTrend_yWeights.png" % (numNodes, radius, rate)
		plt.savefig(oFname, dpi=100)
		plt.clf()

		plt.figure(2)

		plt.yscale('log')
		plt.xticks(locs,labels)
		plt.xlim(plt.xlim()[0], 0.11)
		if (radius == 1.0):
#			plt.ylim(plt.ylim()[0], 1.75)
			plt.ylim(0.005, 10)
		else:
#			plt.ylim(plt.ylim()[0], 20)
			plt.ylim(0.04, 100)

#		plt.plot([-1], [-1], "xk", label="Process 1")
#		plt.plot([-1], [-1], "ok", label="Process 2")

		plt.xlabel(r"$f_v$", fontsize=MEDIUM)
		plt.ylabel(r"Time Constants ($\mu$s)", fontsize=MEDIUM)
		plt.legend(loc=0, fontsize=SMALL, ncol=2)


		plt.tight_layout()
		oFname = "../results/N%dRadius%05.2fRate%5.2fKinetics_CoordinationTrend_tau.png" % (numNodes, radius, rate)
		plt.savefig(oFname, dpi=100)
		plt.clf()

		rIndex += 1
