import sys, os
import numpy as np
from scipy.optimize import curve_fit
from matplotlib import pyplot as plt

import BioNetInput, BioNetTopology

# Fitting
def exp(x, A, B):
	return A * (1 - np.exp(-x/B))

def doubleExp(x, A, B, C, D):
	return A * (1 - np.exp(-x/B)) + C * (1 - np.exp(-x/D))

#
# Graph shit
#

SMALL = 8
MEDIUM = 10
LARGE = 12

#
# Data
#

# Constants
numNodes = 10000

# Variables
vfs = [0.005, 0.01, 0.05, 0.07, 0.1]
ns = [4,6,8,14]
rates = [1000]
radii = [1.0,2.5]

numRuns = 3

# Figure
fig = plt.figure(figsize=(3.2,3.2))

# Loop over variables
for radius in radii:

	for rate in rates:

		# Data output file
		oFname = "../results/N%dRadius%05.2fRate%5.2fKinetics_CrossLinkDensityTrace.dat" % (numNodes, radius, rate)
		fout = open(oFname, "w")

		# Header
		fout.write("Nrates %d Nvolfrac %d Nsite %d\n" % (len(rates), len(vfs), len(ns)))

		for vf in vfs:

			sIndex = 0
			for s in ns:

				# Build data sets
				cldSet = []
				minSize = np.inf
				exists = False

				# Unable to fit for some reason
				if(radius == 1.0 and vf == 0.05 and s == 4):
					numRuns = 1
				else:
					numRuns = 3

				for nr in range(numRuns):

					# Load scripts
					iFname = "../input/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.xml" % (numNodes, s, radius, rate, vf, nr)
					tFname = "../output/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.bntop" % (numNodes, s, radius, rate, vf, nr)
					if(not os.path.exists(tFname)):	
						continue

					inp = BioNetInput.BioNetInput()
					inp.load(iFname)

					top = BioNetTopology.BioNetTopology()
					top.load(tFname)

					# Get data
					bDims = inp.box.dimension
					bVol = bDims[0] * bDims[1] * bDims[2]
					cld = np.array([f.numConnections for f in top.frame]) / bVol
					cldSet.append(cld)
					if cld.size < minSize:
						minSize = cld.size
						time = np.array([f.time for f in top.frame])

					# Flag something existing
					exists = True	


				# Check for existence
				if(not exists):
					continue
				
				# Reduce to minimal size
				time -= time[0]
				time /= 1000.0

				for nr in range(len(cldSet)):
					cldSet[nr] = cldSet[nr][:minSize]

				# Average and errors
				meanTrace = np.mean(cldSet, axis=0)
				sigmaTrace = np.std(cldSet, axis=0)

				# Get rid of identical bits
				for i in range(sigmaTrace.size):
					if sigmaTrace[i] == 0.0:
						sigmaTrace[i] = 1e-10 # Negligible but non-zero

				errTrace = sigmaTrace / np.sqrt(len(cldSet))

				# Add to plot
				plt.errorbar(time[::10], meanTrace[::10], yerr=errTrace[::10], fmt="xC%d" % (sIndex), label=r"$N_s = $%d" % (s))
				

				# And fit (try double exp first)
				cFactor = numNodes / bVol
				func = doubleExp

				if(radius == 1.0 and vf == 0.1 and s == 8):
					guess = np.array([10 * cFactor,100,10 * cFactor,1000])
				else:
					guess = np.array([10 * cFactor,1000,10 * cFactor,1000])

				popt, pcov = curve_fit(func, time, meanTrace, p0=guess, sigma=sigmaTrace)
				perr = np.sqrt(np.fabs(np.diag(pcov)))

				# Test fractional error
				single = False
				for po, pe in zip(popt, perr):
					if(pe/po > 1 or (radius == 2.5 and s == 6 and vf == 0.05)):
						single = True
						func = exp


						guess = np.array([10 * cFactor,1000])
						popt, pcov = curve_fit(func, time, meanTrace, p0=guess, sigma=sigmaTrace)
						perr = np.sqrt(np.fabs(np.diag(pcov)))
						for po2, pe2 in zip(popt, perr):
							if(pe2/po2 > 1):
								popt = []
								pcov = []
								perr = []
							break
						break

				# Sort the fitting paramters out
				if(len(popt) == 0):
					for i in range(4):
						fout.write("None,None,")

				elif(len(popt) == 2):
					for i in range(2):
						fout.write("None,None,")

					for i in range(2):
						fout.write("%e,%e," % (popt[i], perr[i]))
				else:

					# Reorder based on time constants
					if popt[3] < popt[1]:
						temp = popt[1]
						popt[1] = popt[3]
						popt[3] = temp
						temp = perr[1]
						perr[1] = perr[3]
						perr[3] = temp

						temp = popt[0]
						popt[0] = popt[2]
						popt[2] = temp
						temp = perr[0]
						perr[0] = perr[2]
						perr[2] = temp

					for i in range(4):
						fout.write("%e,%e," % (popt[i], perr[i]))

				plt.plot(time, func(time, *popt), "--C%d" % (sIndex))
				
				sIndex += 1

			# Prettify
			plt.ylim(0,8 * cFactor)
			plt.xlabel(r"Time ($\mu$s)", fontsize=MEDIUM)
			plt.ylabel(r"Average Cross-link Density (nm$^{-3}$)", fontsize=MEDIUM)
			plt.legend(loc=0, fontsize=SMALL)

			plt.tight_layout()
			oFname = "../results/N%dRadius%05.2fRate%5.2fvf%0.4fKinetics_CrossLinkDensityTrace.png" % (numNodes, radius, rate, vf)
			plt.savefig(oFname, dpi=100)
			plt.clf()

		fout.close()
