import sys, os
import numpy as np
from matplotlib import pyplot as plt

import BioNetInput, BioNetTrajectory, BioNetTopology

def adjacent_values(vals, q1, q3):
	upper_adjacent_value = q3 + (q3 - q1) * 1.5
	upper_adjacent_value = np.clip(upper_adjacent_value, q3, vals[-1])

	lower_adjacent_value = q1 - (q3 - q1) * 1.5
	lower_adjacent_value = np.clip(lower_adjacent_value, vals[0], q1)
	return lower_adjacent_value, upper_adjacent_value

#
# Graph shit
#
SMALL = 8
MEDIUM = 10
LARGE = 12

#
# Data
#

# Constants
numNodes = 10000

# Variables
vfs = [0.005, 0.01, 0.05, 0.07, 0.1]
ns = [4,6,8,14]
rates = [1000]

radii = [1.0, 2.5]

#vfs = [0.07]
#ns = [8]
#rates = [1000]

#radii = [2.5]

numRuns = 3

# One mega figure
fig = plt.figure(figsize=(3.2,3.2))
ax = plt.gca()

# Master data
boxPositions = range(1,16)

# Loop over variables
for radius in radii:

	for rate in rates:

		sIndex = 0
		for s in ns:

			vIndex = 0
			for vf in vfs:


				# All the data
				data = []

				# Put all pores into a single big distribution
				allForces = np.array([])
				for nr in range(numRuns):

					# Data input
					tFname = "../output/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.bntrj" % (numNodes, s, radius, rate, vf, nr)
					traj = BioNetTrajectory.BioNetTrajectory()
					traj.loadFinalFrame(tFname)
					tpFname = "../output/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.bntop" % (numNodes, s, radius, rate, vf, nr)
					top = BioNetTopology.BioNetTopology()
					top.loadFinalFrame(tpFname)

					# We need to split forces by coordination
					allForces = np.array([np.linalg.norm(f) for f in traj.frame[-1].force])

					maxCoord = np.max(top.frame[-1].nCoord)
					for i in range(0, maxCoord + 1):

						trueArray = top.frame[-1].nCoord == i
						newForces = allForces[trueArray]
						if i < len(data):
							np.append(data[i], newForces)
						else:
							data.append(newForces)



				# Sort zero size arrays
				for i in range(len(data)):
					if data[i].size == 0:
						data[i] = np.array([0])

				# Plot the violin plot for this data
				ax = plt.gca()
				vp = ax.violinplot(data)

				# Colours
				for part, i in zip(vp['bodies'], range(len(data))):
					if s == 4:
						part.set_facecolor('blue')
					elif s == 6:
						part.set_facecolor('orange')
					elif s == 8:
						part.set_facecolor('green')
					elif s == 14:
						part.set_facecolor('red')

					part.set_edgecolor('black')
					part.set_alpha(1)

				# Get quartiles and stuff
				quartile1 = []
				medians = []
				quartile3 = []
				for d in data:
					q1, med, q3 = np.percentile(d, [25, 50, 75])
					quartile1.append(q1)
					medians.append(med)
					quartile3.append(q3)

				whiskers = np.array([adjacent_values(sorted_array, q1, q3) for sorted_array, q1, q3 in zip(data, quartile1, quartile3)])
				whiskersMin, whiskersMax = whiskers[:, 0], whiskers[:, 1]

				inds = np.arange(1, len(medians) + 1)
				ax.scatter(inds, medians, marker='o', color='white', s=30, zorder=3)
				ax.vlines(inds, quartile1, quartile3, color='k', linestyle='-', lw=5)
				ax.vlines(inds, whiskersMin, whiskersMax, color='k', linestyle='-', lw=1)

				# Prettify
				plt.xlabel(r"Cross-link Coordination", fontsize = LARGE)
				plt.ylabel("Force Magnitude (pN)", fontsize = LARGE)

#				plt.xticks([1,2,3,4,5,6,7,8,9,10,11,12], ["0.05","0.07","0.1", "0.05","0.07","0.1","0.05","0.07","0.1","0.05","0.07","0.1"], fontsize=LARGE)
				plt.xticks(range(1, len(data) + 1), [str(i) for i in range(len(data))])
				plt.tight_layout()
				oFname = "../results/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKinetics_ForceDistributionViolin.png" % (numNodes, s, radius, rate, vf)

				plt.savefig(oFname, dpi=100)
				plt.clf()

				vIndex += 1
			sIndex += 1
