import sys, os
import numpy as np
from matplotlib import pyplot as plt

import BioNetInput, BioNetTrajectory

#
# Graph shit
#
SMALL = 8
MEDIUM = 10
LARGE = 12

#
# Data
#

# Constants
numNodes = 10000

# Variables
vfs = [0.005, 0.01, 0.05, 0.07, 0.1]
#vfs = [0.05,0.07,0.1]
ns = [4,6,8,14]
rates = [1000]

radii = [1.0, 2.5]

# Loop over variables
for radius in radii:

	fig = [plt.figure(i, figsize=(3.2,3.2)) for i in range(2)]
	for rate in rates:

		sIndex = 0
		for s in ns:

			vfTrace = []
			fdPwTrace = []
			fdPwErrTrace = []
			fdLinTrace = []
			fdLinErrTrace = []
			turnTrace = []
			turnErrTrace = []
			for vf in vfs:

				# Data input
				iFname = "../results/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKinetics_FractalDimensions.dat" % (numNodes, s, radius, rate, vf)

				# Build data sets
				fdPwSet = []
				fdPwErrSet = []
				fdLinSet = []
				fdLinErrSet = []
				turnSet = []
				turnErrSet = []
				exists = False

				fin = open(iFname, "r")
				data = fin.readline().split()
				fin.close()

				numRuns = int(len(data) / 4)
				for nr in range(numRuns):
					fdPwSet.append(float(data[4 * nr]))
					fdPwErrSet.append(float(data[4 * nr + 1]))
					turnSet.append(float(data[4 * nr + 2]) / radius)
					turnErrSet.append(float(data[4 * nr + 3]) / radius)

					# Flag something existing
					exists = True

				# Check for existence
				if(not exists):
					continue

				# Append
				vfTrace.append(vf)

				# Average and errors
				fdPw = np.mean(fdPwSet)
				fdPwErr = (np.std(fdPwSet) + np.mean(fdPwErrSet)) / np.sqrt(len(fdPwSet))
				turn = np.mean(turnSet)
				turnErr = (np.std(turnSet) + np.mean(turnErrSet)) / np.sqrt(len(turnSet))

				# Append to traces
				fdPwTrace.append(fdPw)
				fdPwErrTrace.append(fdPwErr)
				turnTrace.append(turn)
				turnErrTrace.append(turnErr)

			# Plot each trace
			if(len(vfTrace) != 0):
				plt.figure(0)
				plt.errorbar(vfTrace, fdPwTrace, yerr=fdPwErrTrace, fmt="-xC%d" % (sIndex), label=r"$N_s = $%d" % (s))
#				plt.errorbar(vfTrace, fdLinTrace, yerr=fdLinErrTrace, fmt="-oC%d" % (sIndex))

				plt.figure(1)
				plt.errorbar(vfTrace, turnTrace, yerr=turnErrTrace, fmt="-xC%d" % (sIndex), label=r"$N_s = $%d" % (s))

			sIndex += 1

		# Prettify
		plt.figure(0)
#		plt.xlim(0.04,0.11)
#		plt.xlim(plt.xlim()[0],0.11)
#		plt.ylim(1.9,3.1)
		plt.xlim(0.0,0.11)
		plt.ylim(1.5,2.5)
		plt.xlabel(r"Volume Fraction", fontsize=MEDIUM)
		plt.ylabel("Fractal Dimension", fontsize=MEDIUM)
		plt.legend(loc=4, fontsize=SMALL)

		plt.tight_layout()
		oFname = "../results/N%dRadius%05.2fRate%5.2fKinetics_FractalDimensions.png" % (numNodes, radius, rate)
		plt.savefig(oFname, dpi=100)
	#	plt.show()
		plt.clf()

		plt.figure(1)
#		plt.xlim(0.04,0.11)
#		plt.xlim(plt.xlim()[0],0.11)
		plt.xlim(0.0,0.11)
#		plt.ylim(1.4,2.4)
		if radius == 2.5:
			plt.ylim(4,18)
		else:
			plt.ylim(5,35)
		plt.xlabel(r"Volume Fraction", fontsize=MEDIUM)
		plt.ylabel(r"Lengthscale $(R)$", fontsize=MEDIUM)
		plt.legend(loc=0, fontsize=SMALL)

		plt.tight_layout()
		oFname = "../results/N%dRadius%05.2fRate%5.2fKinetics_FractalDimensionsLength.png" % (numNodes, radius, rate)
		plt.savefig(oFname, dpi=100)
		plt.clf()
