import sys, os
import numpy as np
from matplotlib import pyplot as plt

import BioNetInput, BioNetPores

def adjacent_values(vals, q1, q3):
	upper_adjacent_value = q3 + (q3 - q1) * 1.5
	upper_adjacent_value = np.clip(upper_adjacent_value, q3, vals[-1])

	lower_adjacent_value = q1 - (q3 - q1) * 1.5
	lower_adjacent_value = np.clip(lower_adjacent_value, vals[0], q1)
	return lower_adjacent_value, upper_adjacent_value

#
# Graph shit
#
SMALL = 8
MEDIUM = 10
LARGE = 12

#
# Data
#

# Constants
numNodes = 10000

# Variables
vfs = [0.05, 0.07, 0.1]
ns = [4,6,8,14]
rates = [1000]

radii = [1.0]

numRuns = 3

# One mega figure
fig = plt.figure(figsize=(9,3.2))
ax = plt.gca()

# Master data
boxPositions = range(1,16)

# Loop over variables
for radius in radii:

	for rate in rates:

		# All the data
		data = []
		sIndex = 0
		for s in ns:

			vIndex = 0
			for vf in vfs:

				# Put all pores into a single big distribution
				allRadii = np.array([])
				for nr in range(numRuns):

					# Data input
					iFname = "../output/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fKineticsRun%d.bnpores" % (numNodes, s, radius, rate, vf, nr)

					pores = BioNetPores.BioNetPores()
					pores.load(iFname)

					allRadii = np.append(allRadii, pores.frame[-1].poreRadius)

				data.append(allRadii)
				vIndex += 1
			sIndex += 1


		# Plot the violin plot for this data
		vp = ax.violinplot(data)

		# Colours
		for part, i in zip(vp['bodies'], range(12)):
			if i < 3:
				part.set_facecolor('blue')
			elif i < 6:
				part.set_facecolor('orange')
			elif i < 9:
				part.set_facecolor('green')
			elif i < 12:
				part.set_facecolor('red')

			part.set_edgecolor('black')
			part.set_alpha(1)

		# Get quartiles and stuff
		quartile1 = []
		medians = []
		quartile3 = []
		for d in data:
			q1, med, q3 = np.percentile(d, [25, 50, 75])
			quartile1.append(q1)
			medians.append(med)
			quartile3.append(q3)

		whiskers = np.array([adjacent_values(sorted_array, q1, q3) for sorted_array, q1, q3 in zip(data, quartile1, quartile3)])
		whiskersMin, whiskersMax = whiskers[:, 0], whiskers[:, 1]

		inds = np.arange(1, len(medians) + 1)
		ax.scatter(inds, medians, marker='o', color='white', s=30, zorder=3)
		ax.vlines(inds, quartile1, quartile3, color='k', linestyle='-', lw=5)
		ax.vlines(inds, whiskersMin, whiskersMax, color='k', linestyle='-', lw=1)

		# Prettify
		plt.xlabel("Volume Fraction", fontsize = LARGE)
		plt.ylabel("Pore Radius (nm)", fontsize = LARGE)

		plt.xticks([1,2,3,4,5,6,7,8,9,10,11,12], ["0.05","0.07","0.1", "0.05","0.07","0.1","0.05","0.07","0.1","0.05","0.07","0.1"], fontsize=LARGE)
		plt.tight_layout()
		
		oFname = "../results/N%dRadius%05.2fRate%5.2fKinetics_PoreParametersViolin.png" % (numNodes, radius, rate)

		plt.savefig(oFname, dpi=100)
		plt.clf()
